#!/usr/local/bin/ruby
# bcal10.rb : Blood donors' calendar

# Copyright (C) 1999, Masahiro SATO (m@sa.to)

# Program name
PROGRAM_NAME = <<"ENDOFTEXT"
bcal : Blood donors' calender ver 0.01
  by Masahiro SATO (m@sa.to)
ENDOFTEXT

# require
require 'date2'
require 'bdrec'
require "readline"
include Readline


# constants
TypeName = {
  BloodDonationRecord::T2 => "200ml",
  BloodDonationRecord::T4 => "400ml",
  BloodDonationRecord::PLSPLT => "Plasma & Platelet",
  BloodDonationRecord::PLS => "Plasma",
  BloodDonationRecord::PLT => "Platelet"
}

SexName = {
  BloodDonationRecord::FEMALE => "Female",
  BloodDonationRecord::MALE => "Male"
}


# defaults
$sex = BloodDonationRecord::FEMALE
$bdrec = BloodDonationRecord.new($sex, FALSE, TRUE)

# proc_help
def proc_help
  print PROGRAM_NAME, "\n"
  print "a : add donation record (a YYYY MM DD type)\n"
  print "d : delete donation record (d YYYY MM DD)\n"
  print "l : show donation record (l d or c)\n"
  print "x : change sex (s F or M)\n"
  print "n : show next donation date\n"
  print "r : read file (r FILENAME)\n"
  print "s : save file (s FILENAME)\n"
  print "h : help (this message)\n"
  print "q : quit\n"
end

# proc_add
def proc_add(arg)
  if arg != nil
    y, m, d, t = arg.strip.split(/\s+/, 4)
    if Date.exist3?(y.to_i, m.to_i, d.to_i)
      bd_date = Date.new3(y.to_i, m.to_i, d.to_i)
#      if t =~ /200/
#	$bdrec.add(bd_date, BloodDonationRecord::T2)
#	print bd_date.to_s, " ",
#	  TypeName[BloodDonationRecord::T2], " added\n"
#      elsif t =~ /400/
#	$bdrec.add(bd_date, BloodDonationRecord::T4)
#	print bd_date.to_s, " ",
#	  TypeName[BloodDonationRecord::T4], " added\n"
#      elsif t =~ /PLS/ and t =~ /PLT/
#	$bdrec.add(bd_date, BloodDonationRecord::PLSPLT)
#	print bd_date.to_s, " ",
#	  TypeName[BloodDonationRecord::PLSPLT], " added\n"
#      elsif t =~ /PLS/
#	$bdrec.add(bd_date, BloodDonationRecord::PLS)
#	print bd_date.to_s, " ",
#	  TypeName[BloodDonationRecord::PLS], " added\n"
#      elsif t =~ /PLT/
#	$bdrec.add(bd_date, BloodDonationRecord::PLT)
#	print bd_date.to_s, " ",
#	  TypeName[BloodDonationRecord::PLT], " added\n"
      if t =~ /200/ or t =~ /400/ or t =~ /PLS/i or t =~ /PLT/i
	tp = if t =~ /200/
	       BloodDonationRecord::T2
	     elsif t =~ /400/
	       BloodDonationRecord::T4
	     elsif t =~ /PLS/i and t =~ /PLT/i
	       BloodDonationRecord::PLSPLT
	     elsif t =~ /PLT/i
	       BloodDonationRecord::PLT
	     elsif t =~ /PLS/i
	       BloodDonationRecord::PLS
	     end
	dtype, t2, t4, plt, pls = $bdrec.add(bd_date, tp)
	result = if dtype == tp
		   "added"
		 else
		   "failed to add"
		 end
	print bd_date.to_s, " ", TypeName[tp], " ", result, "\n"
      else
	print "Wrong type\n"
	print "  one of 200ml, 400ml, PLS (stands for plasma)\n"
	print "         PLT (stands for platelet) or PLS & PLT\n"
      end
    else
      print "Wrong format of date\n"
      print "  argument should be \"YYYY MM DD type\"\n"
    end
  else
    print "No argument\n "
    print "  argument should be \"YYYY MM DD type\"\n"
  end    
end

# proc_delete
def proc_delete(arg)
  if arg != nil
    y, m, d = arg.strip.split(/\s+/, 3)
    if Date.exist3?(y.to_i, m.to_i, d.to_i)
      bd_date = Date.new3(y.to_i, m.to_i, d.to_i)
      v = $bdrec.delete(bd_date)
      if v != nil
	print bd_date.to_s, " ", TypeName[v], " deleted\n"
      else
	print "Not deleted\n"
      end
    else
      print "Wrong format of date\n"
      print "  argument should be \"YYYY MM DD\"\n"
    end
  else
    print "No argument\n "
    print "  argument should be \"YYYY MM DD\"\n"
  end    
end

# proc_list
def proc_list(arg)
  torf = { TRUE => "T", FALSE => "F" }
  if arg =~ /^c/i
    if $bdrec.changed?
      $bdrec.calc_exec
      $bdrec.calc_cannot
      $bdrec.calc_stop
    end
    cannot_list = $bdrec.cannot_list
    cannot_list.keys.sort.each {|d|
      print d.to_s
      for i in 0..3
	print " ", torf[cannot_list[d][i]]
      end
      print "\n"
    }
  elsif arg =~ /^d/i
    dona_list = $bdrec.dona_list
    dona_list.keys.sort.each {|d|
      print d.to_s, " ", TypeName[dona_list[d]], "\n"
    }
  else
    print "Wrong argument\n"
    print "  argument should be C or D\n"
    print "  D : List of donation date\n"
    print "  C : List of date when cannot donate\n"
  end
end

# proc_sex
def proc_sex(arg)
  if arg =~ /^f/i
    $sex = BloodDonationRecord::FEMALE
    $bdrec.change_sex($sex)
  elsif arg =~ /^m/i
    $sex = BloodDonationRecord::MALE
    $bdrec.change_sex($sex)
  else
    print "No or wrong argument\n"
    print "  argument should be F (Female) or M (Male)\n"
  end
#  print "You are ", SexName[$sex], "\n"
  print "You are ", SexName[$bdrec.sex], "\n"
end

# proc_next
#   'next donation date' means the next date of
#    the largest 'unable-to-donate' date
def proc_next
  if $bdrec.changed?
    $bdrec.calc_exec
    $bdrec.calc_cannot
    $bdrec.calc_stop
  end
  cannot_list = $bdrec.cannot_list
  if cannot_list.length == 0
    print "You can donate your blood right now :-)\n"
  else
#    bd = []
#    for i in 0..3
#      cannot_list.keys.sort.each {|d|
#	bd[i] = d + 1 if !cannot_list[d][i]
#      }
#    end
    n = cannot_list.keys.sort[cannot_list.length - 1] + 1
    cannot_list[n] = [TRUE, TRUE, TRUE, TRUE]
    bd = []
    bd_flag = []
    for i in 0..3
      bd_flag[i] = FALSE
    end
    cannot_list.keys.sort{|xx,yy| yy<=>xx}.each {|d|
      for i in 0..3
	if !bd_flag[i]
	  if cannot_list[d][i]
	    bd[i] = d
	  else
	    bd_flag[i] = TRUE
	  end
	end
	break if bd_flag[0] and bd_flag[1] and bd_flag[2] and bd_flag[3]
      end
    }
    print "Your next donation date is\n"
    print "  ", TypeName[BloodDonationRecord::T2], "\t", bd[0], "\n"
    print "  ", TypeName[BloodDonationRecord::T4], "\t", bd[1], "\n"
    print "  ", TypeName[BloodDonationRecord::PLT], "\t", bd[2], "\n"
    print "  ", TypeName[BloodDonationRecord::PLS], "\t", bd[3], "\n"
  end
end

# proc_read
def proc_read(arg)
  begin
    open(arg.strip, "r") {|f|
      print "Reading  \"", arg, "\"\n"
      $sex = f.gets.chop.to_i # 1st line
      $bdrec = BloodDonationRecord.new($sex, FALSE, TRUE) # new :-)
      print "You are ", SexName[$bdrec.sex], "\n"
#      $bdrec.calc_stop # stop for a moment
      f.each_line {|line| # date, type
	y, m, d, tp = line.chop.split
	bd_date = Date.new3(y.to_i, m.to_i, d.to_i)
	dtype, t2, t4, plt, pls =
	  $bdrec.add(bd_date, tp.to_i)
	result = if dtype == tp.to_i
		   "added"
		 else
		   "failed to add"
		 end
	print bd_date.to_s, " ", TypeName[tp.to_i], " ", result, "\n"
      }
#      $bdrec.calc_exec # allow to calc
#      $bdrec.calc_cannot # calc
    }
  rescue
    print "Cannot open file \"", arg, "\" for read\n"
  end
end

# proc_save
def proc_save(arg)
  begin
    open(arg.strip, "w") {|f|
      f.print $bdrec.sex, "\n"
      dona_list = $bdrec.dona_list
      dona_list.keys.sort.each {|d|
#	f.print d.year, " ", d.mon, " ", d.mday, " ", dona_list[d], "\n"
	f.printf("%d %2d %2d %d\n", d.year, d.mon, d.mday, dona_list[d])
      }
      print "Saved to \"", arg, "\"\n"
    }
  rescue
    print "Cannot open file \"", arg, "\" for write\n"
  end
end

# proc_quit
def proc_quit
  print "See you again (^o^)/~~\n\n"
end

# main loop
while TRUE
  # prompt
#  print "\nbcal> "
#  STDOUT.flush
#  cmd, arg = gets.chop.split(/\s+/, 2)

  print "\n"
  cmd, arg = readline("bcal> ", TRUE).strip.split(/\s+/, 2)

  # procedure
  if cmd =~ /^h/i
    proc_help
  elsif cmd =~ /^a/i
    proc_add(arg)
  elsif cmd =~ /^d/i
    proc_delete(arg)
  elsif cmd =~ /^l/i
    proc_list(arg)
  elsif cmd =~ /^x/i
    proc_sex(arg)
  elsif cmd =~ /^n/i
    proc_next
  elsif cmd =~ /^r/i
    proc_read(arg)
  elsif cmd =~ /^s/i
    proc_save(arg)
  elsif cmd =~ /^q/i
    proc_quit
    break
  else
    print "Sorry, I don't know. Try \"h\" for help.\n"
  end
end


