#!/usr/local/bin/ruby
# tkbcal10.rb : Blood donors' calender using Tk

# Copyright (C) 1999, Masahiro SATO (m@sa.to)

# Program name =========================================================
PROGRAM_NAME = <<"ENDOFTEXT"
tkbcal.rb : Blood donors' calender using Ruby/Tk, ver 0.01
by Masahiro SATO (m@sa.to)
ENDOFTEXT

# require ==============================================================
require 'date2'
require 'bdrec'
require 'tk'
require 'tkscrollbox'
begin require 'holiday' rescue; end

# constants ============================================================
NORM_FORE = 'Black'
NORM_BACK = 'LightBlue'
NORM_AFORE = 'Black'
NORM_ABACK = 'SkyBlue'

HOLI_FORE = 'Red'
HOLI_AFORE = 'Red'

WEND_FORE = 'DarkGreen'
WEND_AFORE = 'DarkGreen'

DONA_BACK = 'Pink'
DONA_ABACK = 'HotPink'

CANNOT_BACK = 'LightGrey'
CANNOT_ABACK = 'Grey'
CANNOT_AT_ALL_BACK = 'LightSlateGrey'
CANNOT_AT_ALL_ABACK = 'SlateGrey'

TypeName = {
  BloodDonationRecord::T2 => "200",
  BloodDonationRecord::T4 => "400",
  BloodDonationRecord::PLSPLT => "PnP",
  BloodDonationRecord::PLS => "PLS",
  BloodDonationRecord::PLT => "PLT",
  BloodDonationRecord::DONOT => "Don\'t"
}

ShortTypeName = ["2", "4", "T", "S", "-"]

SexName = {
  BloodDonationRecord::FEMALE => "Female",
  BloodDonationRecord::MALE => "Male"
}

MyWeek = ['Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat']

DayButtonWidth = 5

# defaults =============================================================
$bdrec = BloodDonationRecord.new(BloodDonationRecord::FEMALE)
$sheet_list = []
$list_list = []

# TkInfoDialog =========================================================
class TkInfoDialog < TkDialog
  def initialize(keys = nil)
    super(keys)
  end

  def title
    return "Infomation"
  end

  def message
    return PROGRAM_NAME
  end

  def bitmap
    return "info"
  end

  def default_button
    return 0
  end

  def buttons
    return ["OK"]
  end
end

# TkChangeDialog =======================================================
class TkChangeDialog < TkDialog
  def initialize(date = Date.today, b = ["Cancel"], keys = nil)
    @date = date
    @title = date.to_s
    @buttons = b
    super(keys)
  end

  def title
    return @title
  end

  def message
    return "What do you like?"
  end

  def bitmap
    return "questhead"
    # man Tk_NameOfBitmap
  end

  def default_button
    return 0
  end

  def buttons
    return @buttons
  end
end

# refresh_screen =======================================================
def refresh_screen
  $sheet_list.each {|s|
    s.refresh
  }
  $list_list.each {|s|
    s.refresh
  }
end

# change_dona_type =====================================================
def change_dona_type(button_id, date = Date.today)
  type, t2, t4, plt, pls = $bdrec.dona_type(date)
  name = ["Cancel"]
  num = [0]

  # donation day
  if type != nil
    name.push(TypeName[BloodDonationRecord::DONOT])
    num.push(BloodDonationRecord::DONOT)
  end

  # cannot donate?
  if t2
    name.push(TypeName[BloodDonationRecord::T2])
    num.push(BloodDonationRecord::T2)
  end
  if t4
    name.push(TypeName[BloodDonationRecord::T4])
    num.push(BloodDonationRecord::T4)
  end
  if plt
    name.push(TypeName[BloodDonationRecord::PLT])
    num.push(BloodDonationRecord::PLT)
  end
  if pls
    name.push(TypeName[BloodDonationRecord::PLS])
    num.push(BloodDonationRecord::PLS)
  end
  if plt and pls
    name.push(TypeName[BloodDonationRecord::PLSPLT])
    num.push(BloodDonationRecord::PLSPLT)
  end

  # check
  retval = TkChangeDialog.new(date, name).value
  if retval != 0
    $bdrec.add(date, num[retval]) 
    refresh_screen
  end
#  print retval, " ", num[retval], "\n"
end

# TkDayButton ==========================================================
class TkDayButton < TkButton
  def initialize(date = Date.today, parent = nil, keys = nil)
    super(parent, keys)
    @date = date
    set(@date)
    width DayButtonWidth
    command proc{change_dona_type(self, @date)}
  end

  # set text of button
  def set(date = @date)
    # change date
    @date = date if @date != date

    # set fore color
    if ($".include?("holiday.rb") and @date.nhol?) or @date.wday == 0
      foreground HOLI_FORE
      activeforeground HOLI_AFORE
    elsif @date.wday == 6
      foreground WEND_FORE
      activeforeground WEND_AFORE
    else
      foreground NORM_FORE
      activeforeground NORM_AFORE
    end

    # check status
    type, t2, t4, plt, pls = $bdrec.dona_type(@date)

    # make text
    str = date.mday.to_s + "\n" +
      case type
      when BloodDonationRecord::T2, BloodDonationRecord::T4,
	  BloodDonationRecord::PLSPLT, BloodDonationRecord::PLS,
	  BloodDonationRecord::PLT
	TypeName[type]
      when nil
#	if t2 then "2" else "-" end +
#	  if t4 then "4" else "-" end +
#	  if plt then "T" else "-" end +
#	  if pls then "S" else "-" end
	if t2 then ShortTypeName[0] else ShortTypeName[4] end +
	  if t4 then ShortTypeName[1] else ShortTypeName[4] end +
	  if plt then ShortTypeName[2] else ShortTypeName[4] end +
	  if pls then ShortTypeName[3] else ShortTypeName[4] end
      end
    text str

    # background color
    case type
    when BloodDonationRecord::T2, BloodDonationRecord::T4,
	BloodDonationRecord::PLSPLT, BloodDonationRecord::PLS,
	BloodDonationRecord::PLT
      background DONA_BACK
      activebackground DONA_ABACK
    when nil
      if t2 and t4 and plt and pls
	background NORM_BACK
	activebackground NORM_ABACK
      elsif t2 or t4 or plt or pls
	background CANNOT_BACK
	activebackground CANNOT_ABACK
      else
	background CANNOT_AT_ALL_BACK
	activebackground CANNOT_AT_ALL_ABACK
      end
    end
  end

end

# TkWeek ===============================================================
class TkWeek < TkLabel
  def initialize(num = 0, parent = nil, keys = nil)
    super(parent, keys)
    text MyWeek[num]
    if num == 0
      foreground HOLI_FORE
    elsif num == 6
      foreground WEND_FORE
    else
      foreground NORM_FORE
    end
    background NORM_BACK
  end
end

# TkCalender ===========================================================
class TkCalender < TkFrame
  def initialize(ym = YearMonth.new, parent = nil, keys = nil)
    super(parent, keys)
    @ym = ym
    @day_list = []
    set_cal
    background NORM_BACK
  end

  def set_cal(ym = @ym)
    @ym = ym if @ym != ym
    @day_list.each {|d|
      d.destroy
    }
    @day_list.clear

    # day of week
    for i in 0..6
      TkWeek.new(i, self) {
		       grid "row"=>0, "column"=>i, "sticky"=>"ew"
      }
    end

    # date
    current = Date.new3(@ym.year, @ym.month, 1)
    r = 1
    while current.mon == @ym.month
      @day_list.push(TkDayButton.new(current, self) {
		       grid "row"=>r, "column"=>current.wday, "sticky"=>"ew"
		     })
      current += 1
      r += 1 if current.wday == 0 # Sunday
    end
  end

  def refresh
    @day_list.each {|d|
      d.set
    }
  end
end

# TkButtonInSheet ======================================================
class TkButtonInSheet < TkButton
  def initialize(parent = nil, keys = nil)
    super(parent, keys)
    foreground NORM_FORE
    activeforeground NORM_AFORE
    background NORM_BACK
    activebackground NORM_ABACK
  end
end

# TkCalenderSheet ======================================================
class TkCalenderSheet < TkToplevel
  def initialize(ym = YearMonth.new, parent = nil, keys = nil)
    super(parent, keys)
    @ym = ym
    title @ym.to_s

    # calender
    @cal = TkCalender.new(ym, self).pack "side"=>"bottom"

    # back
    TkButtonInSheet.new(self,
			"text" => "<- Prev -",
			"command" =>  proc {
			  @ym.prev!
			  self.title @ym.to_s
			  @cal.set_cal(@ym)
			}
			).pack "side"=>"left", "fill"=>"both", "expand"=>"yes"

    # fore
    TkButtonInSheet.new(self,
			"text" =>  "- Next ->",
			"command" =>  proc {
			  @ym.next!
			  self.title @ym.to_s
			  @cal.set_cal(@ym)
			}
			).pack "side"=>"right", "fill"=>"both", "expand"=>"yes"
    # close
#    if $sheet_list.length > 0
      TkButtonInSheet.new(self,
			  "text" =>  "Close",
			  "command" =>  proc {
			    $sheet_list.delete(self)
			    self.destroy
			  }
			  ).pack "side"=>"left",
	                         "fill"=>"both", "expand"=>"yes"
    end
#  end

  def refresh
    @cal.refresh
  end

  def set(ym = @ym)
    @ym = ym
    self.title @ym.to_s
    @cal.set_cal(@ym)
  end
end

# YearMonth ============================================================
class YearMonth
  def initialize(y = Date.today.year, m = Date.today.mon)
    @year = y
    m = 1 if m < 1
    m = 12 if m > 12
    @month = m
  end

  def year
    return @year
  end

  def month
    return @month
  end

  def to_s
    return format("%d-%02d", @year, @month)
  end

  def next!
    @month += 1
    if @month > 12
      @year += 1
      @month = 1
    end
  end

  def prev!
    @month -= 1
    if @month < 1
      @year -= 1
      @month = 12
    end
  end

  def next
    return YearMonth.new(@year, @month).next!
  end

  def prev
    return YearMonth.new(@year, @month).prev!
  end
end

# TkMainListButton =====================================================
class TkMainListButton < TkButton
  def initialize(parent = nil, keys = nil)
    super(parent, keys)
    foreground NORM_FORE
    background NORM_BACK
    activeforeground NORM_AFORE
    activebackground NORM_ABACK
    width 20
  end
end

# TkQuitDialog =========================================================
class TkQuitDialog < TkDialog
  def initialize(keys = nil)
    super(keys)
  end

  def title
    return "Quit?"
  end

  def message
    return "Do you want to quit?"
  end

  def bitmap
    return "questhead"
  end

  def buttons
    ["Yes", "No"]
  end

  def default_button
    return 1
  end
end

# quit_check ===========================================================
def quit_check(root_id)
  if TkQuitDialog.new.value == 0
    root_id.destroy
  end
end

# TkQuit ===============================================================
class TkQuit < TkMainListButton
  def initialize(root_id, parent = nil, keys = nil)
    super(parent, keys)
    text "Quit"
    command proc {quit_check(root_id)}
  end
end

# TkSexChange ==========================================================
class TkSexChange < TkMainListButton
  def initialize(sex = BloodDonationRecord::FEMALE, parent = nil, keys = nil)
    super(parent, keys)
    change(sex)
  end

  def change(sex = @sex)
    @sex = sex
    $bdrec.change_sex(@sex)
    text SexName[@sex]
    command proc {
      if @sex == BloodDonationRecord::FEMALE
	@sex = BloodDonationRecord::MALE
      elsif @sex == BloodDonationRecord::MALE
	@sex = BloodDonationRecord::FEMALE
      else
	@sex = BloodDonationRecord::FEMALE
      end
      $bdrec.change_sex(@sex)
      refresh_screen
      text SexName[@sex]
    }
  end

  def sex
    return @sex
  end
end

# TkInfoButton =========================================================
class TkInfoButton < TkMainListButton
  def initialize(parent = nil, keys = nil)
    super(parent, keys)
    text "Information"
    command proc {TkInfoDialog.new}
  end
end

# TkDonaList ===========================================================
class TkDonaList < TkToplevel
  def initialize(parent = nil, keys = nil)
    super(parent, keys)
    title "List"
    @date = []
    @scrlbox = TkScrollbox.new(self) {
      width 18
      height 10
      pack "side"=>"top"
    }
    refresh
    TkButton.new(self,
		 "text" => "Close",
		 "command" => proc {
		   $list_list.delete(self)
		   self.destroy
		 }
		 ).pack "side"=>"top", "fill"=>"both", "expand"=>"yes"
  end

  def refresh
    @date.clear
    @scrlbox.delete(0, "end")
    $bdrec.dona_list.keys.sort.each {|d|
      @date.push(YearMonth.new(d.year, d.mon))
      type, t2, t4, plt, pls = $bdrec.dona_type(d)
      @scrlbox.insert("end", d.to_s + "   " + TypeName[type])
    }
    @scrlbox.bind("Double-Button-1", proc {
		    if $sheet_list[0] != nil
		      $sheet_list[0].set(@date[@scrlbox.index("active").to_i])
		    end
		  })
  end
end

# TkReadSaveFile =======================================================
class TkReadSaveFile < TkToplevel
  def initialize(type = FileReadSave::READ, parent = nil, keys = nil)
    super(parent, keys)

    if type == FileReadSave::READ
      @read = TRUE
    else
      @read = FALSE
    end

    if @read
      title "Read file"
    else 
      title "Save file"
    end

    self.grab
    self.focus

    entry = TkEntry.new(self,
			"width"=>"20",
			).pack "side"=>"top", "fill"=>"x", "expand"=>"yes"
    entry.value = $file.name
    ok = TkButton.new(self,
		      "text"=>"OK",
		      "command"=>proc {
			if @read
			  $file.read(entry.value.strip)
			  refresh_screen
			else
			  $file.save(entry.value.strip)
			end
			self.grab "release"
			self.destroy
		      }
		      ).pack "side"=>"left", "fill"=>"both", "expand"=>"yes"
    can = TkButton.new(self,
		       "text"=>"Cancel",
		       "command"=>proc {
			 self.grab "release"
			 self.destroy
		       }
		       ).pack "side"=>"left", "fill"=>"both", "expand"=>"yes"
    entry.bind("Return", proc {ok.invoke})
  end
end

# TkNewSheet ===========================================================
class TkNewSheet < TkMainListButton
  def initialize(parent = nil, keys = nil)
    super(parent, keys)
    text "New Calender"
    command proc {$sheet_list.push(TkCalenderSheet.new)}
  end
end

# TkNewList ============================================================
class TkNewList < TkMainListButton
  def initialize(parent = nil, keys = nil)
    super(parent, keys)
    text "New List"
    command proc {$list_list.push(TkDonaList.new)}
  end
end

# TkReadFile ===========================================================
class TkReadFile < TkMainListButton
  def initialize(parent = nil, keys = nil)
    super(parent, keys)
    text "Read File"
    command proc {
#      $file.read
      TkReadSaveFile.new(FileReadSave::READ)
#      refresh_screen
    }
  end
end

# TkSaveFile ===========================================================
class TkSaveFile < TkMainListButton
  def initialize(parent = nil, keys = nil)
    super(parent, keys)
    text "Save File"
    command proc {
#      $file.save
      TkReadSaveFile.new(FileReadSave::SAVE)
#      refresh_screen
    }
  end
end

# TkBCal ===============================================================
class TkBCal < TkRoot
  def initialize(parent = nil, keys = nil)
    super(parent, keys)
    title "tkbcal.rb"

    [$sex_button = TkSexChange.new(BloodDonationRecord::FEMALE, self),
      TkReadFile.new(self),
      TkSaveFile.new(self),
      TkNewSheet.new(self),
      TkNewList.new(self),
      TkInfoButton.new(self),
      TkQuit.new(self, self)].each {|b|
      b.pack "side"=>"top", "fill"=>"both", "expand"=>"yes"
    }

    $sheet_list.push(TkCalenderSheet.new)
    $list_list.push(TkDonaList.new)
  end
end

# FileReadSave =========================================================
class FileReadSave
  READ, SAVE = 0, 1

  def initialize(fname = "dona.dat")
    @filename = fname
    @saved = TRUE
  end

  def name
    return @filename
  end

  def read(fname = @filename)
    @filename = fname if @filename != fname
    begin
      open(@filename, "r") {|f|
	sex = f.gets.chop.to_i # 1st line
	$bdrec = BloodDonationRecord.new(sex) # new :-)
	$sex_button.change(sex)
	$bdrec.calc_stop # stop for a moment
	f.each_line {|line| # date, type
	  y, m, d, tp = line.chop.split
	  bd_date = Date.new3(y.to_i, m.to_i, d.to_i)
	  dtype, t2, t4, plt, pls = $bdrec.add(bd_date, tp.to_i)
	}
	$bdrec.calc_exec # allow to calc
	$bdrec.calc_cannot # calc
      }
    rescue
#      print "Cannot open file \"", arg, "\" for read\n"
    end

    @saved = TRUE
  end

  def save(fname = @filename)
    @filename = fname if @filename != fname
    begin
      open(@filename, "w") {|f|
	f.print $bdrec.sex, "\n"
	dona_list = $bdrec.dona_list
	dona_list.keys.sort.each {|d|
	  f.printf("%d %2d %2d %d\n", d.year, d.mon, d.mday, dona_list[d])
	}
      }
    rescue
#      print "Cannot open file \"", arg, "\" for write\n"
    end
    
    @saved = TRUE
  end

  def saved?
    return @saved
  end
end

$file = FileReadSave.new

# main =================================================================
TkBCal.new

# loop =================================================================
Tk.mainloop
